% Power and channel resource allocation for F/TDMA relay network
% Direct transmission (DT) and decode and forward (DF)

function [Rate_Joint, Pm, Ps, Pr, dual_P, Qm, Qr, dual_T, Power_Feasible] = power_channel_df(M, N, C, RDF_enable, gsd, gsr, grd, beta, T_min, Num_JointIter, step_dual_P, c, P_max, Pmk, Psk, Prk, dual_P, Qm, Qr, dual_T)

Num_Link = size(M,1);
gsd_tmp = gsd*ones(1,N);

Pm = zeros(Num_Link,1);
Ps = zeros(Num_Link,N);
Pr = zeros(Num_Link,N);

% Parameters for the optimization algorithm
Num_AuxilIter = 2;
Num_PowerIter = 1;
min_dual_P = 1e-6;

Rate_Joint = zeros(Num_Link, Num_JointIter*Num_AuxilIter);
Power_Feasible = zeros(N, Num_JointIter*Num_AuxilIter);

iter_joint = 1;
while (iter_joint<=Num_JointIter) % channel adjust every 10 iterations

    % Power allocation optimization for fixed channel allocation
    iter_auxil = 1;
    while (iter_auxil<=Num_AuxilIter) % channel allocation every 2 iterations
%         display(iter_auxil)

        iter_power = 1;
        while (iter_power<=Num_PowerIter)
%             display(iter_power)
            Power_used = zeros(N,1);
            
            % Optimal power given channel allocation and dual variables
            for m=1:Num_Link
                Pm(m,1) = one_dt_link_proximal(gsd(m,1), Qm(m,1), dual_P(M(m,1),1), c, Pmk(m,1));
                Power_used(M(m,1),1) = Power_used(M(m,1),1) + Pm(m,1);
                Jm = (1:1:N).*RDF_enable(m,:);
                Jm = Jm(Jm>0);
                for i = 1:length(Jm)
                    [Ps(m,Jm(i)), Pr(m,Jm(i))] = one_df_link_proximal( gsd(m,1),grd(m,Jm(i)),gsr(m,Jm(i)),Qr(m,Jm(i)),dual_P(M(m,1)),dual_P(Jm(i)),c,Psk(m,Jm(i)),Prk(m,Jm(i)));
                    Power_used(M(m,1),1) = Power_used(M(m,1),1) + Ps(m,Jm(i));
                    Power_used(Jm(i),1) = Power_used(Jm(i),1) + Pr(m,Jm(i));
                end
            end

%             dual_P = dual_P + step_dual_P*((1+temp_m)/(iter_power+temp_m))*(Power_used-P_max);
            dual_P = dual_P + step_dual_P*(Power_used-P_max);
            dual_P = max(dual_P,min_dual_P);
            iter_power = iter_power + 1;
        end

        Pmk = Pm;
        Psk = Ps;
        Prk = Pr;

        object_direct = Qm.*log2(1+gsd.*Pm./Qm);
        object_relay = Qr./2.*min(log2(1+2*gsd_tmp.*Ps./Qr+2*grd.*Pr./Qr),log2(1+2*gsr.*Ps./Qr));
        object_direct(isnan(object_direct))=0;
        object_relay(isnan(object_relay))=0;
        Rate_Joint(:,(iter_joint-1)*Num_AuxilIter+iter_auxil) = object_direct + sum(object_relay,2);
        Power_Feasible(:,(iter_joint-1)*Num_AuxilIter+iter_auxil) = (Power_used-P_max);        
        iter_auxil = iter_auxil + 1;
    end  

    % Channel allocation optimization for fixed power allocation
    for index_control=1:1:length(beta)
        e = 1;
        e_low = 0;
        e_up = 0; % 50 is the maximal value f can be (when a/x is bixmax in Matlab), e_up should be smaller than 50/2
        tmp_T = 0;
        tmp_link = (1:1:Num_Link)'.*(C==index_control);
        tmp_link = tmp_link(tmp_link>0);
        iter_channel = 0;
        while ((abs(tmp_T-beta(index_control))>=1e-6)&&(e-e_low>1e-6))
            tmp_T = 0;
            for index_link = 1:1:length(tmp_link)
                m = tmp_link(index_link);
                Qm(m) = max(f_line(gsd(m)*Pm(m,1),e), T_min);
                tmp_T = tmp_T + Qm(m);
                Jm = (1:1:N).*RDF_enable(m,:);
                Jm = Jm(Jm>0);
                for j = 1:1:length(Jm)
                    Qr(m,Jm(j)) = max(f_line(2*min(gsd(m)*Ps(m,Jm(j))+grd(m,Jm(j))*Pr(m,Jm(j)), gsr(m,Jm(j))*Ps(m,Jm(j))),2*e), T_min);
                    tmp_T = tmp_T + Qr(m,Jm(j));
                end
            end

            if (tmp_T>beta(index_control))
                e_low = e;
                if (e_up==0)
                    e = e*2;
                else
                    e = (e_low+e_up)/2;
                end
            elseif (tmp_T<beta(index_control))
                e_up = e;
                e = (e_low+e_up)/2;  
                if (e<1e-20) 
                    e = 0;
                    break;
                end
            end
            iter_channel = iter_channel + 1;
        end
        dual_T(index_control,1) = e;
    end
    
    iter_joint = iter_joint + 1;

end